from django.contrib import admin
from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _
from mapwidgets import GooglePointFieldWidget
from sorl.thumbnail.admin import AdminImageMixin

from inventor.core.bookings.admin import BookingMixinAdmin
from inventor.core.listings.models.general import Album, Video, Photo
from inventor.core.listings.models.listing_types import Accommodation, Property, EatAndDrink, Service, Vacation, Event, Shop, Vehicle, Profile, Job, Course, \
    Nature


class AlbumInline(admin.StackedInline):
    model = Album
    extra = 1


class VideoInline(admin.StackedInline):
    model = Video
    extra = 1


class ListingAdmin(AdminImageMixin, admin.ModelAdmin):
    date_hierarchy = 'created'
    search_fields = ['title', 'description']
    list_display = ('id', 'title', 'slug', 'address', 'location', 'created')
    list_display_links = ('title',)
    # list_filter = ('location', 'country', )
    autocomplete_fields = ['author', 'location']

    fieldsets = (
        (_('Definition'), {'fields': ('title', 'slug', 'description',)}),
        (_('Management'), {'fields': ('author', 'published', 'promoted')}),
        (_('Specification'), {'fields': (('categories', 'features'),)}),
        (_('Price'), {'fields': (('price_starts_at', 'price', 'price_unit', 'price_per_person'),)}),
        (_('Address'), {'fields': ('location', 'country', 'address', 'point')}),
        (_('Previews'), {'fields': (('image', 'banner'),)}),
        (_('Contact information'), {'fields': ('person', 'phone', 'email', 'website')}),
        (_('Social connections'), {'fields': ('social_networks',)}),
    )
    filter_vertical = ['categories', 'features']  # TODO: filter categories for selected listing type only
    inlines = [VideoInline, AlbumInline]  # add comments?
    formfield_overrides = {
        models.PointField: {"widget": GooglePointFieldWidget}
    }

    class Media:
        js = (
            'js/map-widget-utils.js',
        )


class PhotoInline(admin.StackedInline):
    model = Photo
    extra = 1


@admin.register(Album)
class AlbumAdmin(admin.ModelAdmin):
    search_fields = ['listing__title', 'title', 'description']
    list_display = ('listing', 'title', 'description')
    raw_id_fields = ['listing']
    inlines = [PhotoInline]


# Listing types

@admin.register(Property, EatAndDrink, Service, Vacation, Event, Shop, Vehicle, Profile, Job, Course, Nature)
class ListingTypeAdmin(ListingAdmin):
    pass


@admin.register(Accommodation)
class AccommodationAdmin(ListingAdmin):
    fieldsets = ListingTypeAdmin.fieldsets + BookingMixinAdmin.fieldsets + (
        (_('Specific'), {'fields': ('amenities', 'star_rating', 'rooms')}),
    )



from django.contrib.auth.models import UserManager as DjangoUserManager
from django.db import models
from django.db.models import Q


class UserQuerySet(models.QuerySet):
    def active(self):
        return self.filter(is_active=True)

    def with_perm(self, perm_name):
        return self.filter(
            Q(is_superuser=True) |
            Q(user_permissions__codename=perm_name) |
            Q(groups__permissions__codename=perm_name)
        ).distinct()


class UserManager(DjangoUserManager):
    def get_queryset(self):
        return UserQuerySet(self.model, using=self._db)

    def _create_user(self, email, password, **extra_fields):
        """
        Creates and saves a User with the given username, email and password.
        """
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email=None, password=None, **extra_fields):
        extra_fields.setdefault('is_staff', False)
        extra_fields.setdefault('is_superuser', False)
        return self._create_user(email, password, **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        return self._create_user(email, password, **extra_fields)

    def get_by_full_name(self, full_name, ignore_accent=False):
        first_name, last_name = full_name.split(' ', 1)
        first_name = first_name.strip()
        last_name = last_name.strip()

        if ignore_accent:
            return self.get(first_name__unaccent=first_name, last_name__unaccent=last_name)
        return self.get(first_name=first_name, last_name=last_name)

    def filter_by_full_name(self, full_name):
        try:
            first_name, last_name = full_name.split(' ', 1)
            first_name = first_name.strip()
            last_name = last_name.strip()
            return self.filter(first_name__unaccent__icontains=first_name, last_name__unaccent__icontains=last_name)
        except ValueError:
            return self.filter(Q(first_name__unaccent__icontains=full_name) | Q(last_name__unaccent__icontains=full_name))

    def active(self):
        return self.get_queryset().active()

    def with_perm(self, perm_name):
        return self.get_queryset().with_perm(perm_name)



# Generated by Django 2.2.4 on 2020-03-09 09:56

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('listings', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Booking',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('AWAITING_APPROVAL', 'Awaiting approval'), ('PENDING_PAYMENT', 'Pending payment'), ('APPROVED', 'Approved'), ('CANCELLED', 'Cancelled'), ('NO_SHOW', 'No-show'), ('COMPLETED', 'Completed')], default='AWAITING_APPROVAL', max_length=17, verbose_name='status')),
                ('hash_key', models.CharField(db_index=True, max_length=40, unique=True, verbose_name='hash key')),
                ('book_from', models.DateTimeField(blank=True, default=None, null=True, verbose_name='book from')),
                ('book_to', models.DateTimeField(blank=True, default=None, null=True, verbose_name='book to')),
                ('number_of_persons', models.PositiveSmallIntegerField(blank=True, default=None, null=True, verbose_name='number of persons')),
                ('price', models.DecimalField(blank=True, decimal_places=2, default=None, help_text='€', max_digits=10, null=True, validators=[django.core.validators.MinValueValidator(0)], verbose_name='price')),
                ('message', models.TextField(blank=True, verbose_name='message')),
                ('person', models.CharField(blank=True, max_length=100, verbose_name='person')),
                ('phone', models.CharField(blank=True, max_length=40, verbose_name='phone')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='modified')),
                ('listing', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='listings.Listing', verbose_name='listing')),
                ('traveler', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='traveler')),
            ],
            options={
                'verbose_name': 'booking',
                'verbose_name_plural': 'bookings',
                'ordering': ('created',),
            },
        ),
    ]



from django.contrib import admin
from mptt.admin import DraggableMPTTAdmin

from inventor.core.lexicons.models import Location, Category, Feature
from inventor.core.lexicons.models import AccommodationAmenity


@admin.register(Category)
class CategoryAdmin(DraggableMPTTAdmin):
    search_fields = ['title']
    list_display = ('tree_actions', 'indented_title', 'slug', 'listing_type')
    list_display_links = ('indented_title',)
    list_select_related = ['listing_type']


@admin.register(Location)
class LocationAdmin(DraggableMPTTAdmin):
    search_fields = ['title']
    list_display = ('tree_actions', 'indented_title', 'slug')
    list_display_links = ('indented_title',)


@admin.register(Feature)
class FeatureAdmin(admin.ModelAdmin):
    search_fields = ['title']
    list_display = ('title', 'slug')


# Regular Lexicons

@admin.register(
    AccommodationAmenity,
    # AccommodationType, PropertyType
)
class RegularLexiconAdmin(admin.ModelAdmin):
    search_fields = ['title']
    list_display = ('title', 'slug', 'description')



# Generated by Django 2.2.4 on 2020-03-09 13:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('listings', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='accommodation',
            name='booking_url',
            field=models.URLField(blank=True, max_length=300, verbose_name='booking URL'),
        ),
        migrations.AddField(
            model_name='eatanddrink',
            name='booking_url',
            field=models.URLField(blank=True, max_length=300, verbose_name='booking URL'),
        ),
    ]



import django_filters
from crispy_forms.bootstrap import InlineRadios
from crispy_forms.layout import Layout
from django.db import models
from django import forms
from django.utils.translation import ugettext_lazy as _

from inventor.core.lexicons.models import Category
from inventor.core.listings.models.general import Listing
from inventor.utils import SingleSubmitFormHelper, PositiveBooleanFilter


class ListingFilter(django_filters.FilterSet):
    keyword = django_filters.CharFilter(label=_('Keyword'), method=lambda qs, name, value: qs.by_keyword(value))
    # price = SliderFilter(label=_('Price'), min_value=0, max_value=1000, step=10, appended_text=' €', has_range=True, segment='listings.Listing.price')  # TODO
    price = django_filters.RangeFilter(label=_('Price'), help_text=_('€'))
    # categories = django_filters.MultipleChoiceFilter(choices=Vehicle.TYPES, widget=forms.CheckboxSelectMultiple)

    class Meta:
        model = Listing
        fields = [
            'keyword',
            'promoted',
            'location',
            'price',
            'categories', 'features'
        ]
        filter_overrides = {
            models.CharField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            },
            models.TextField: {
                'filter_class': django_filters.CharFilter,
                'extra': lambda f: {
                    'lookup_expr': 'icontains',
                },
            },
            models.BooleanField: {
                'filter_class': PositiveBooleanFilter,
                'extra': lambda f: {
                    'widget': forms.CheckboxInput,
                },
            },
        }

    def __init__(self, listing_type=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = SingleSubmitFormHelper()
        self.helper.form_tag = False
        self.helper.disable_csrf = True
        self.helper.layout = Layout(
            'keyword',
            'promoted',
            'location',
            'price',
            'categories', 'features'
        )
        self.form.fields['location'].empty_label = _('All locations')

        # dynamic categories
        self.filters['categories'].field_name = 'categories__slug'

        if listing_type:
            self.form.fields['categories'].queryset = Category.objects.of_listing_type(listing_type).exclude(parent=None)
        else:
            self.form.fields['categories'].queryset = Category.objects.filter(parent=None)

        self.form.fields['categories'] = forms.MultipleChoiceField(
            label=_('Categories'),
            choices=list(self.form.fields['categories'].queryset.values_list('slug', 'title')),  # TODO: cache?
            required=False,
            widget=forms.CheckboxSelectMultiple
        )

        # changed filter logic of features
        self.filters['features'].method = 'filter_features'

    def filter_features(self, queryset, name, value):
        # return listings having ALL features, not AT LEAST one
        for feature in value:
            queryset = queryset.filter(features=feature)
        return queryset



def get_model():
    from inventor.contrib.comments.models import ReviewComment
    return ReviewComment


def get_form():
    from inventor.contrib.comments.forms import CommentForm
    return CommentForm



# Generated by Django 2.2.4 on 2020-03-09 09:56

from django.db import migrations, models
import django.db.models.deletion
import inventor.core.listings.mixins
import mptt.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='AccommodationAmenity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, unique=True, verbose_name='title')),
                ('slug', models.SlugField(default='', max_length=150, unique=True)),
                ('description', models.TextField(blank=True, verbose_name='description')),
            ],
            options={
                'verbose_name': 'accommodation amenity',
                'verbose_name_plural': 'accommodation amenities',
                'ordering': ('title',),
            },
            bases=(inventor.core.listings.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Feature',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, unique=True, verbose_name='title')),
                ('slug', models.SlugField(default='', max_length=150, unique=True)),
            ],
            options={
                'verbose_name': 'feature',
                'verbose_name_plural': 'features',
                'ordering': ('title',),
            },
            bases=(inventor.core.listings.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, unique=True, verbose_name='title')),
                ('slug', models.SlugField(default='', max_length=150, unique=True)),
                ('description', models.TextField(blank=True, verbose_name='description')),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='lexicons.Location')),
            ],
            options={
                'verbose_name': 'location',
                'verbose_name_plural': 'locations',
                'ordering': ('title',),
            },
            bases=(inventor.core.listings.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, verbose_name='title')),
                ('slug', models.SlugField(default='', max_length=150, unique=True)),
                ('lft', models.PositiveIntegerField(editable=False)),
                ('rght', models.PositiveIntegerField(editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(editable=False)),
                ('listing_type', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='contenttypes.ContentType', verbose_name='listing type')),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='lexicons.Category')),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
                'ordering': ('title',),
                'unique_together': {('title', 'listing_type')},
            },
            bases=(inventor.core.listings.mixins.SlugMixin, models.Model),
        ),
    ]



# Generated by Django 2.2.4 on 2020-03-09 09:56

from django.conf import settings
import django.contrib.gis.db.models.fields
import django.contrib.postgres.fields.hstore
import django.contrib.postgres.fields.ranges
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import internationalflavor.countries.models
import inventor.core.listings.mixins


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('lexicons', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Album',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=100, verbose_name='title')),
                ('description', models.TextField(blank=True, verbose_name='description')),
            ],
            options={
                'verbose_name': 'album',
                'verbose_name_plural': 'albums',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='Listing',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100, verbose_name='title')),
                ('slug', models.SlugField(max_length=150, unique=True)),
                ('description', models.TextField(blank=True, verbose_name='description')),
                ('published', models.BooleanField(default=True, verbose_name='published')),
                ('promoted', models.BooleanField(default=False, verbose_name='promoted')),
                ('price_starts_at', models.BooleanField(default=False, verbose_name='price starts at')),
                ('price', models.DecimalField(blank=True, db_index=True, decimal_places=2, default=None, help_text='€', max_digits=10, null=True, validators=[django.core.validators.MinValueValidator(0)], verbose_name='price')),
                ('price_unit', models.CharField(blank=True, choices=[('ENTRY', 'entry'), ('HOUR', 'hour'), ('DAY', 'day'), ('NIGHT', 'night')], max_length=5, verbose_name='price per unit')),
                ('price_per_person', models.BooleanField(default=False, verbose_name='price per person')),
                ('address', models.TextField(blank=True, help_text='street, postcode, city', max_length=500, verbose_name='address')),
                ('country', internationalflavor.countries.models.CountryField(blank=True, db_index=True, verbose_name='country')),
                ('point', django.contrib.gis.db.models.fields.PointField(blank=True, db_index=True, default=None, null=True, srid=4326, verbose_name='point')),
                ('image', models.ImageField(blank=True, help_text='photo or image', max_length=1024, upload_to='images', verbose_name='image')),
                ('banner', models.ImageField(blank=True, help_text='photo or image', max_length=5120, upload_to='banners', verbose_name='banner')),
                ('person', models.CharField(blank=True, max_length=100, verbose_name='person')),
                ('phone', models.CharField(blank=True, max_length=40, verbose_name='phone')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email')),
                ('website', models.URLField(blank=True, max_length=400, verbose_name='website')),
                ('social_networks', django.contrib.postgres.fields.hstore.HStoreField(blank=True, default=dict, verbose_name='social networks')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='modified')),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='author')),
                ('categories', models.ManyToManyField(blank=True, related_name='listings_of_category', to='lexicons.Category', verbose_name='categories')),
                ('features', models.ManyToManyField(blank=True, related_name='listings_with_features', to='lexicons.Feature', verbose_name='features')),
                ('location', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='lexicons.Location')),
            ],
            options={
                'verbose_name': 'listing',
                'verbose_name_plural': 'listings',
                'db_table': 'listings_general',
                'ordering': ('title',),
            },
            bases=(inventor.core.listings.mixins.SlugMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Course',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'course',
                'verbose_name_plural': 'courses',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='EatAndDrink',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
                ('bookings_enabled', models.BooleanField(default=True, verbose_name='bookings enabled')),
                ('booking_period', models.CharField(blank=True, choices=[('HOUR', 'hour'), ('DAY', 'day'), ('NIGHT', 'night')], max_length=5, verbose_name='booking period')),
                ('bookings_min_period', models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='min period')),
                ('bookings_max_period', models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='max period')),
                ('bookings_min_persons', models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='min persons')),
                ('capacity', models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='capacity')),
            ],
            options={
                'verbose_name': 'eat & drink',
                'verbose_name_plural': 'eat & drinks',
                'ordering': ('title',),
            },
            bases=('listings.listing', models.Model),
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
                ('date', django.contrib.postgres.fields.ranges.DateTimeRangeField(blank=True, db_index=True, default=None, null=True, verbose_name='datetime range')),
            ],
            options={
                'verbose_name': 'event',
                'verbose_name_plural': 'events',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Job',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'job',
                'verbose_name_plural': 'jobs',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Nature',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'animal or plant',
                'verbose_name_plural': 'animals and plants',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'profile',
                'verbose_name_plural': 'profiles',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Property',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'property',
                'verbose_name_plural': 'properties',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Service',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'service',
                'verbose_name_plural': 'services',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Shop',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'shop',
                'verbose_name_plural': 'shops',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Vacation',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'vacation',
                'verbose_name_plural': 'vacations',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Vehicle',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'vehicle',
                'verbose_name_plural': 'vehicle',
                'ordering': ('title',),
            },
            bases=('listings.listing',),
        ),
        migrations.CreateModel(
            name='Video',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(blank=True, max_length=100, verbose_name='title')),
                ('description', models.TextField(blank=True, verbose_name='description')),
                ('url', models.URLField(max_length=300, verbose_name='URL')),
                ('listing', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='listings.Listing')),
            ],
            options={
                'verbose_name': 'video',
                'verbose_name_plural': 'videos',
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='Photo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.ImageField(help_text='photo, image or icon', max_length=5120, upload_to='photos', verbose_name='file')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', models.DateTimeField(auto_now=True, verbose_name='modified')),
                ('album', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='listings.Album')),
            ],
            options={
                'verbose_name': 'photo',
                'verbose_name_plural': 'photos',
                'ordering': ('created',),
            },
        ),
        migrations.AddField(
            model_name='album',
            name='listing',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='listings.Listing'),
        ),
        migrations.CreateModel(
            name='Accommodation',
            fields=[
                ('listing_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='listings.Listing')),
                ('bookings_enabled', models.BooleanField(default=True, verbose_name='bookings enabled')),
                ('booking_period', models.CharField(blank=True, choices=[('HOUR', 'hour'), ('DAY', 'day'), ('NIGHT', 'night')], max_length=5, verbose_name='booking period')),
                ('bookings_min_period', models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='min period')),
                ('bookings_max_period', models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='max period')),
                ('bookings_min_persons', models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='min persons')),
                ('capacity', models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='capacity')),
                ('star_rating', models.SmallIntegerField(blank=True, default=None, null=True, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(5)], verbose_name='Star rating')),
                ('rooms', models.SmallIntegerField(blank=True, default=None, null=True, validators=[django.core.validators.MinValueValidator(1)], verbose_name='number of rooms')),
                ('amenities', models.ManyToManyField(blank=True, to='lexicons.AccommodationAmenity', verbose_name='amenities')),
            ],
            options={
                'verbose_name': 'accommodation',
                'verbose_name_plural': 'accommodations',
                'ordering': ('title',),
            },
            bases=('listings.listing', models.Model),
        ),
    ]



from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models
from django.utils.translation import ugettext_lazy as _
from threadedcomments.models import ThreadedComment


class ReviewComment(ThreadedComment):
    RATING_MIN = 1
    RATING_MAX = 5
    TYPES = [
        ('COMMENT', _('comment')),
        ('REVIEW', _('review'))
    ]

    type = models.CharField(_('type'), choices=TYPES, max_length=7)
    rating = models.PositiveSmallIntegerField(_('rating'),
        validators=[MinValueValidator(RATING_MIN), MaxValueValidator(RATING_MAX)],
        blank=True, null=True, default=None)



from django.urls import path
from django.utils.text import slugify
from django.utils.translation import pgettext_lazy, ugettext_lazy

from inventor.core.listings.models.general import Listing
from inventor.core.listings.views import ListingListView, ListingDetailView

app_name = 'listings'

urlpatterns = [
    path(pgettext_lazy('url', 'listings/'), ListingListView.as_view(), name='listing_list'),
]

# lists
for listing_type in Listing.__subclasses__():
    url_name = listing_type.get_list_url_name()
    verbose_name_plural = ugettext_lazy(listing_type._meta.verbose_name_plural)
    url_path = slugify(verbose_name_plural) + '/'
    urlpatterns.append(path(url_path, ListingListView.as_view(template_name='listings/listing_list.html', model=listing_type), name=url_name))

# listing detail
urlpatterns.append(path(pgettext_lazy('url', '<str:slug>/'), ListingDetailView.as_view(), name='listing_detail'))



from threadedcomments.templatetags.threadedcomments_tags import RenderCommentListNode

from django import template

register = template.Library()


class RelatedRenderCommentListNode(RenderCommentListNode):
    def get_queryset(self, context):
        qs = super().get_queryset(context)
        return qs.select_related('user')


@register.tag
def render_comment_list(parser, token):
    return RelatedRenderCommentListNode.handle_token(parser, token)



from django.db.models import F
from django.views.generic import TemplateView

from inventor.core.listings.models.listing_types import Accommodation


class HomeView(TemplateView):
    template_name = 'home.html'

    def get_context_data(self, **kwargs):
        context_data = super().get_context_data(**kwargs)
        context_data.update({
            'top_accommodations': Accommodation.objects
                # .promoted()
                .select_subclasses()
                .only('id', 'slug', 'title', 'promoted',
                      'location_id', 'location__title',
                      'image', 'price', 'price_unit', 'price_starts_at')
                .annotate(location_title=F('location__title'))
                .order_by('?')[:6]
        })
        return context_data



# Generated by Django 2.2.4 on 2020-03-09 13:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('listings', '0002_auto_20200309_1403'),
    ]

    operations = [
        migrations.RenameField(
            model_name='accommodation',
            old_name='bookings_enabled',
            new_name='booking_enabled',
        ),
        migrations.RenameField(
            model_name='accommodation',
            old_name='bookings_max_period',
            new_name='booking_max_period',
        ),
        migrations.RenameField(
            model_name='accommodation',
            old_name='bookings_min_period',
            new_name='booking_min_period',
        ),
        migrations.RenameField(
            model_name='accommodation',
            old_name='bookings_min_persons',
            new_name='booking_min_persons',
        ),
        migrations.RenameField(
            model_name='eatanddrink',
            old_name='bookings_enabled',
            new_name='booking_enabled',
        ),
        migrations.RenameField(
            model_name='eatanddrink',
            old_name='bookings_max_period',
            new_name='booking_max_period',
        ),
        migrations.RenameField(
            model_name='eatanddrink',
            old_name='bookings_min_period',
            new_name='booking_min_period',
        ),
        migrations.RenameField(
            model_name='eatanddrink',
            old_name='bookings_min_persons',
            new_name='booking_min_persons',
        ),
    ]



from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout
from threadedcomments.forms import ThreadedCommentForm


class CommentForm(ThreadedCommentForm):
    def __init__(self, *args, **kwargs):
        super(CommentForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.layout = Layout(
            'content_type', 'object_pk', 'parent',
            'honeypot', 'timestamp', 'security_hash',
            #'name', 'email', 'url', 'title',
            'comment', 'rating'
        )
        self.fields['comment'].label = ''
        self.fields['comment'].widget.attrs['rows'] = 3



from django.contrib.postgres.operations import HStoreExtension, UnaccentExtension, CreateExtension
from django.db import migrations


class Migration(migrations.Migration):
    # unaccent, pg_trgm, hstore;

    operations = [
        UnaccentExtension(),
        CreateExtension('pg_trgm'),
        HStoreExtension(),
    ]



from django.contrib.auth.models import Permission, Group
from crispy_forms.bootstrap import FormActions, InlineRadios, PrependedAppendedText, AppendedText, PrependedText
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Fieldset, Submit, Div, Row, HTML
from django import forms
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from django_select2.forms import ModelSelect2Widget, ModelSelect2MultipleWidget

from permissions_widget.forms import PermissionSelectMultipleField
from permissions_widget.layout import PermissionWidget
from inventor.core.accounts.models import User


class UserWidget(ModelSelect2Widget):
    queryset = User.objects.all()
    search_fields = ['first_name__icontains', 'last_name__icontains']
    # data_view = 'accounts:user_select2'

    # def __init__(self, *args, **kwargs):
    #     defaults = {'data_view': self.data_view}
    #     defaults.update(kwargs)
    #     super().__init__(*args, **defaults)

    def filter_queryset(self, term, queryset=None, **dependent_fields):
        if queryset is None:
            queryset = self.get_queryset()
        if term.isdigit():
            return queryset.filter(pk=term).distinct()
        return super().filter_queryset(term, queryset, **dependent_fields)


class UsersWidget(ModelSelect2MultipleWidget):
    queryset = User.objects.all()
    search_fields = ['first_name__icontains', 'last_name__icontains']
    # data_view = 'accounts:user_select2'

    # def __init__(self, *args, **kwargs):
    #     defaults = {'data_view': self.data_view}
    #     defaults.update(kwargs)
    #     super().__init__(*args, **defaults)

    def filter_queryset(self, term, queryset=None, **dependent_fields):
        if queryset is None:
            queryset = self.get_queryset()
        if term.isdigit():
            return queryset.filter(pk=term).distinct()
        return super().filter_queryset(term, queryset, **dependent_fields)


class UpdateProfileForm(forms.ModelForm):
    class Meta:
        fields = (
            'first_name', 'last_name', 'email', 'phone', 'avatar'
        )
        model = User

    def __init__(self, *args, **kwargs):
        super(UpdateProfileForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.layout = Layout(
            Row(
                Fieldset(
                    _('Personal details'),
                    Row(
                        Div('first_name', css_class='col-md-5'),
                        Div('last_name', css_class='col-md-7'),
                    ),
                    'avatar',
                    css_class='col-md-7'
                ),
                Fieldset(
                    _('Contact details'),
                    PrependedText('email', '<i class="fas fa-at"></i>'),
                    PrependedText('phone', '<i class="far fa-mobile"></i>'),
                    css_class='col-md-5'
                ),
            ),
            FormActions(
                Submit('submit', _('Submit'), css_class='btn-secondary')
            )
        )


class UserForm(forms.ModelForm):
    user_permissions = PermissionSelectMultipleField(required=False)

    class Meta:
        model = User
        fields = (
            'first_name', 'last_name', 'email', 'phone', 'avatar',
            'is_active', 'is_superuser', 'is_staff',
            'groups', 'user_permissions',
        )

    def __init__(self, *args, **kwargs):
        super(UserForm, self).__init__(*args, **kwargs)

        groups_permissions = []
        if self.instance and self.instance.pk:
            groups_permissions = Permission.objects.filter(group__user=self.instance)

        self.fields['is_superuser'].disabled = True
        self.fields['is_staff'].disabled = True

        self.helper = FormHelper()
        self.helper.layout = Layout(
            Row(
                Fieldset(
                    _('Personal details'),
                    Row(
                        Div('first_name', css_class='col-md-5'),
                        Div('last_name', css_class='col-md-7'),
                    ),
                    'avatar',
                    css_class='col-md-4'
                ),
                Fieldset(
                    _('Contact details'),
                    PrependedText('email', '<i class="fas fa-at"></i>'),
                    PrependedText('phone', '<i class="far fa-mobile"></i>'),
                    css_class='col-md-3'
                ),
                Fieldset(
                    _('Classification'),
                    'groups',
                    css_class='col-md-3'
                ),
                Fieldset(
                    _('Status'),
                    'is_active',
                    'is_superuser',
                    'is_staff',
                    css_class='col-md-5'
                ),
            ),
            Row(
                Fieldset(
                    _('Permissions'),
                    PermissionWidget('user_permissions', groups_permissions=groups_permissions),
                    css_class='col-md-12'
                )
            ),
            FormActions(
                Submit('submit', _('Submit'), css_class='btn-secondary')
            )
        )



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as DjangoUserAdmin
from django.contrib.auth.models import Permission
from django.contrib.humanize.templatetags.humanize import naturaltime
from django.utils.translation import ugettext_lazy as _
from inventor.core.accounts.models import User


@admin.register(User)
class UserAdmin(DjangoUserAdmin):
    list_display = ('email', 'first_name', 'last_name', 'phone', 'is_staff', 'is_superuser')
    search_fields = ('first_name', 'last_name', 'email')
    add_fieldsets = (
        (_('Password'), {'fields': ('password1', 'password2')}),
        (_('Login'), {'fields': ('email',)}),
    )
    fieldsets = (
        (None, {'fields': ('password',)}),
        (_('Personal info'), {'fields': ('first_name', 'last_name', 'email', 'avatar')}),
        (_('Contact'), {'fields': ('phone',)}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser', 'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
        # (_('Notifications'), {'fields': ('notices_settings',)}),
    )
    ordering = ('last_name', 'first_name')


@admin.register(Permission)
class PermissionAdmin(admin.ModelAdmin):
    list_display = ('pk', 'name', 'model_display', 'app_label', 'codename')
    search_fields = ('name', 'codename')
    list_select_related = ('content_type',)

    def app_label(self, obj):
        return obj.content_type.app_label

    def model_display(self, obj):
        return obj.content_type.model



import os
from django.conf import settings
from django.contrib.contenttypes.fields import GenericRelation
from django.contrib.postgres.fields import HStoreField
from django.contrib.gis.db import models
from django.contrib.gis.db.models import Avg
from django.core.validators import MinValueValidator
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from django_comments import get_model as get_comment_model
from internationalflavor.countries import CountryField
from sorl import thumbnail

from inventor import settings as inventor_settings
from inventor.core.lexicons.models import Category, Feature, Location
from inventor.core.listings.managers import ListingQuerySet


# TODO: opening hours, meals and drinks, street view, faq
from inventor.core.listings.mixins import SlugMixin


class Listing(SlugMixin, models.Model):
    FORCE_SLUG_REGENERATION = False

    SOCIAL_NETWORKS = ['Facebook', 'Twitter', 'Google', 'Instagram', 'Vimeo', 'YouTube', 'LinkedIn', 'Dribbble',
                       'Skype', 'Foursquare', 'Behance']  # TODO: move to settings

    PRICE_UNITS = [
        ('ENTRY', _('entry')),                    # tickets
        ('HOUR', _('hour')),                      # parking
        ('DAY', _('day')),                        # vehicle per day
        ('NIGHT', _('night')),                    # room unit per night
    ]

    # definition
    title = models.CharField(_('title'), max_length=100)
    slug = models.SlugField(unique=True, max_length=SlugMixin.MAX_SLUG_LENGTH)
    description = models.TextField(_('description'), blank=True)

    # management
    author = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, verbose_name=_('author'))
    published = models.BooleanField(_('published'), default=True)
    promoted = models.BooleanField(_('promoted'), default=False)

    # specification
    categories = models.ManyToManyField(to=Category, verbose_name=_('categories'), blank=True, related_name='listings_of_category')
    features = models.ManyToManyField(to=Feature, verbose_name=_('features'), blank=True, related_name='listings_with_features')

    # price
    price_starts_at = models.BooleanField(_('price starts at'), default=False)
    price = models.DecimalField(_('price'), help_text=inventor_settings.CURRENCY, max_digits=10, decimal_places=2, db_index=True, validators=[MinValueValidator(0)],
                                blank=True, null=True, default=None)
    price_unit = models.CharField(_('price unit'), choices=PRICE_UNITS, max_length=5, blank=True)
    price_per_person = models.BooleanField(_('price per person'), default=False)

    # address
    location = models.ForeignKey(Location, on_delete=models.SET_NULL,
                                 blank=True, null=True, default=None)

    address = models.TextField(_('address'), help_text=_('street, postcode, city'), max_length=500, blank=True)

    # street = models.CharField(_('street'), max_length=200, blank=True)
    # postcode = models.CharField(_('postcode'), max_length=30, blank=True)
    # city = models.CharField(_('city'), max_length=50, blank=True)
    country = CountryField(verbose_name=_('country'), blank=True, db_index=True)
    point = models.PointField(_('point'), blank=True, null=True, default=None, db_index=True)

    # previews
    image = thumbnail.ImageField(
        verbose_name=_('image'),
        help_text=_('photo or image'),
        max_length=1024,
        upload_to='images',
        blank=True
    )

    banner = models.ImageField(
        verbose_name=_('banner'),
        help_text=_('photo or image'),
        max_length=1024 * 5,
        upload_to='banners',
        blank=True
    )

    # contact information
    person = models.CharField(_('person'), max_length=100, blank=True)
    phone = models.CharField(_('phone'), max_length=40, blank=True)
    email = models.EmailField(_('email'), blank=True)
    website = models.URLField(_('website'), max_length=400, blank=True)

    # social
    social_networks = HStoreField(verbose_name=_('social networks'), blank=True, default=dict)

    # relations
    comments = GenericRelation(get_comment_model(), content_type_field='content_type', object_id_field='object_pk',
                               related_query_name='comment')

    created = models.DateTimeField(_('created'), auto_now_add=True)
    modified = models.DateTimeField(_('modified'), auto_now=True)
    objects = ListingQuerySet.as_manager()

    class Meta:
        verbose_name = _('listing')
        verbose_name_plural = _('listings')
        ordering = ('title',)
        db_table = 'listings_general'

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('listings:listing_detail', args=(self.slug,))

    def get_update_url(self):
        return reverse('listings:listing_update', args=(self.pk,))

    @property
    def full_address(self):
        return '{}, {}'.format(self.address, self.country).strip(', ')

    @property
    def get_price_display(self):
        if not self.price:
            return ''

        price = str(self.price)

        if price.endswith('.00'):
            price = price[:-3]

        if inventor_settings.CURRENCY_AFTER_AMOUNT:
            price_display = '{}{}'.format(price, inventor_settings.CURRENCY)
        else:
            price_display = '{}{}'.format(inventor_settings.CURRENCY, price)

        if self.price_starts_at:
            price_display = '{} {}'.format(_('starts at'), price_display)

        if self.price_unit:
            price_display = '{} / {}'.format(price_display, self.get_price_unit_display())

        return price_display

    @cached_property
    def rating(self):
        avg_rating = self.comments.aggregate(Avg('rating'))
        # TODO: rating subjects:
        # - Facilities / Amenities
        # - Cleanliness
        # - Comfort
        # - Location
        # - Services?
        rating = avg_rating['rating__avg']
        if rating:
            rating = round(rating, 2)
        return rating

    def delete(self, **kwargs):
        """ Deletes file before deleting instance """
        self.delete_banner()
        super().delete(**kwargs)

    def delete_banner(self):
        """ Deletes image file """
        try:
            os.remove(self.banner.path)
        except ValueError:
            pass
        except IOError:
            pass
        except OSError:
            pass

    @property
    def listing_class(self):
        return self.__class__

    def get_listing_type_display(self):
        return self.listing_class._meta.verbose_name

    def get_listing_type_display_plural(self):
        return self.listing_class._meta.verbose_name_plural

    @classmethod
    def get_list_url_name(cls):
        url_name = f'{cls.__name__.lower()}_list'
        return f'{url_name}'

    @classmethod
    def get_list_url(cls):
        url_name = cls.get_list_url_name()
        return reverse(f'listings:{url_name}')

    def get_real_instance(self):
        """ get object child instance """

        def get_subclasses(cls):
            subclasses = cls.__subclasses__()
            result = []
            for subclass in subclasses:
                if not subclass._meta.abstract:
                    result.append(subclass)
                else:
                    result += get_subclasses(subclass)
            return result

        if hasattr(self, '_real_instance'):  # try looking in our cache
            return self._real_instance

        subclasses = get_subclasses(self.__class__)

        if not subclasses:  # already real_instance
            self._real_instance = self
            return self._real_instance
        else:
            subclasses_names = [cls.__name__.lower() for cls in subclasses]
            for subcls_name in subclasses_names:
                if hasattr(self, subcls_name):
                    return getattr(self, subcls_name, self)
            return self


class Video(models.Model):
    listing = models.ForeignKey(Listing, on_delete=models.CASCADE)
    title = models.CharField(_('title'), max_length=100, blank=True)
    description = models.TextField(_('description'), blank=True)
    url = models.URLField(_('URL'), max_length=300)

    class Meta:
        verbose_name = _('video')
        verbose_name_plural = _('videos')
        ordering = ('title', )

    def __str__(self):
        return self.title if self.title else self.url


class Album(models.Model):
    listing = models.ForeignKey(Listing, on_delete=models.CASCADE)
    title = models.CharField(_('title'), max_length=100, blank=True)
    description = models.TextField(_('description'), blank=True)

    class Meta:
        verbose_name = _('album')
        verbose_name_plural = _('albums')
        ordering = ('title',)

    def __str__(self):
        return self.title


class Photo(models.Model):
    album = models.ForeignKey(Album, on_delete=models.CASCADE)
    file = models.ImageField(
        verbose_name=_('file'),
        help_text=_('photo, image or icon'),
        max_length=1024 * 5,
        upload_to='photos',  # TODO: listing folder
    )
    created = models.DateTimeField(_('created'), auto_now_add=True)
    modified = models.DateTimeField(_('modified'), auto_now=True)

    class Meta:
        verbose_name = _(u'photo')
        verbose_name_plural = _(u'photos')
        ordering = ('created',)

    def __unicode__(self):
        return self.file.name

    def delete(self, **kwargs):
        """ Deletes file before deleting instance """
        self.delete_file()
        super(Photo, self).delete(**kwargs)

    def delete_file(self):
        """ Deletes image file """
        try:
            os.remove(self.file.path)
        except ValueError:
            pass
        except IOError:
            pass
        except OSError:
            pass

    def get_absolute_url(self):
        return reverse('inventor:photo_download', kwargs={'pk': self.pk})



# Generated by Django 2.2.4 on 2019-08-08 13:13

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('threadedcomments', '0003_threadedcomment_newest_activity'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReviewComment',
            fields=[
                ('threadedcomment_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='threadedcomments.ThreadedComment')),
                ('type', models.CharField(choices=[('COMMENT', 'comment'), ('REVIEW', 'review')], max_length=7, verbose_name='type')),
                ('rating', models.PositiveSmallIntegerField(blank=True, default=None, null=True, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(5)], verbose_name='rating')),
            ],
            options={
                'verbose_name': 'comment',
                'verbose_name_plural': 'comments',
                'ordering': ('submit_date',),
                'permissions': [('can_moderate', 'Can moderate comments')],
                'abstract': False,
            },
            bases=('threadedcomments.threadedcomment',),
        ),
    ]



from django.contrib.postgres.fields import DateTimeRangeField
from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models
from django.utils.translation import ugettext_lazy as _
from inventor.core.bookings.mixins import BookingMixin
from inventor.core.lexicons.models import AccommodationAmenity
from inventor.core.listings.models.general import Listing


class Accommodation(BookingMixin, Listing):
    # Apartmán
    # Chata v prírode
    # Dovolenkový dom
    # Horská chata
    # Hotel
    # Hostel
    # Penzión
    # Privát
    # Resort

    section = _('travel')  # ?
    amenities = models.ManyToManyField(to=AccommodationAmenity, verbose_name=_('amenities'), blank=True)
    # type = models.ForeignKey( # TODO: Deprecated - replaced by categories
    #     AccommodationType, verbose_name=_('type'), on_delete=models.SET_NULL, related_name='type',
    #     blank=True, null=True, default=None)
    star_rating = models.SmallIntegerField(
        verbose_name=_('Star rating'), validators=[MinValueValidator(1), MaxValueValidator(5)],
        blank=True, null=True, default=None)
    rooms = models.SmallIntegerField(
        verbose_name=_('number of rooms'), validators=[MinValueValidator(1)],
        blank=True, null=True, default=None)

    class Meta:
        verbose_name = _('accommodation')
        verbose_name_plural = _('accommodations')
        ordering = ('title',)


class Property(Listing):
    section = _('real estate')
    # TODO: type: business, house, flat, area?

    class Meta:
        verbose_name = _('property')
        verbose_name_plural = _('properties')
        ordering = ('title',)


class EatAndDrink(BookingMixin, Listing):  # TODO
    section = _('gastronomy')
    # type: restaurant, cafe, pub, bistro, fast-food

    class Meta:
        verbose_name = _('eat & drink')
        verbose_name_plural = _('eat & drinks')
        ordering = ('title',)


class Service(Listing):
    section = _('services')
    # barber, taxi, haircut, shop

    class Meta:
        verbose_name = _('service')
        verbose_name_plural = _('services')
        ordering = ('title',)


class Vacation(Listing):  # Relax/Vacation
    section = _('tourism')

    class Meta:
        verbose_name = _('vacation')
        verbose_name_plural = _('vacations')
        ordering = ('title',)


class Event(Listing):
    section = _('events')
    # concert, speech, conference

    date = DateTimeRangeField(_('datetime range'), db_index=True,
                              blank=True, null=True, default=None)

    class Meta:
        verbose_name = _('event')
        verbose_name_plural = _('events')
        ordering = ('title',)


class Shop(Listing):  # Store
    section = _('shopping')

    class Meta:
        verbose_name = _('shop')
        verbose_name_plural = _('shops')
        ordering = ('title',)


class Vehicle(Listing):
    section = _('automotive')

    class Meta:
        verbose_name = _('vehicle')
        verbose_name_plural = _('vehicle')
        ordering = ('title',)


class Profile(Listing):
    section = _('dating')

    class Meta:
        verbose_name = _('profile')
        verbose_name_plural = _('profiles')
        ordering = ('title',)


class Job(Listing):
    section = _('work')

    class Meta:
        verbose_name = _('job')
        verbose_name_plural = _('jobs')
        ordering = ('title',)


class Course(Listing):
    section = _('education')

    class Meta:
        verbose_name = _('course')
        verbose_name_plural = _('courses')
        ordering = ('title',)


class Nature(Listing):
    section = _('flora and fauna')
    #type = animal, plant

    class Meta:
        verbose_name = _('animal or plant')
        verbose_name_plural = _('animals and plants')
        ordering = ('title',)



from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from inventor.core.bookings.models import Booking


@admin.register(Booking)
class BookingAdmin(admin.ModelAdmin):
    date_hierarchy = 'created'
    list_display = ('id', 'listing', 'traveler', 'created')
    list_display_links = ('id',)
    autocomplete_fields = ['traveler']


class BookingMixinAdmin(admin.ModelAdmin):
    fieldsets = (
        (_('Booking'), {'fields': ('booking_enabled', ('booking_period', 'booking_min_period', 'booking_max_period'), ('booking_min_persons', 'capacity'))}),
    )



import hashlib
import random
from datetime import datetime

import django_filters
from crispy_forms.helper import FormHelper


def generate_hash(length=16):
    random_number = str(random.random())
    current_data = str(datetime.now())
    salt = hashlib.sha1(random_number.encode('utf-8')).hexdigest()
    pepper = hashlib.sha1(current_data.encode('utf-8')).hexdigest()
    digest = hashlib.sha1((salt + pepper).encode('utf-8')).hexdigest()
    return digest[:length]


class SingleSubmitFormHelper(FormHelper):  # TODO: move to django-pragmatic
    def __init__(self, form=None):
        super().__init__(form)
        self.attrs['onsubmit'] = "submit.disabled=true; return true;"


class PositiveBooleanFilter(django_filters.BooleanFilter):   # TODO: move to django-pragmatic
    def filter(self, qs, value):
        if not value:
            return qs
        return super().filter(qs, value)



# Generated by Django 2.2.4 on 2019-08-08 13:12

from django.db import migrations, models
import django.utils.timezone
import inventor.core.accounts.managers
import inventor.core.accounts.utils


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('first_name', models.CharField(blank=True, db_index=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, db_index=True, max_length=30, verbose_name='last name')),
                ('email', models.EmailField(max_length=254, unique=True, verbose_name='email')),
                ('phone', models.CharField(blank=True, max_length=30, verbose_name='phone')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('last_seen', models.DateTimeField(blank=True, default=None, null=True, verbose_name='last seen')),
                ('avatar', models.ImageField(blank=True, help_text='photo, image or icon', max_length=1024, upload_to=inventor.core.accounts.utils.avatar_path_handler, verbose_name='avatar')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'ordering': ('first_name', 'last_name', 'email'),
                'default_permissions': ('list', 'view', 'add', 'change', 'delete'),
            },
            managers=[
                ('objects', inventor.core.accounts.managers.UserManager()),
            ],
        ),
    ]



from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.shortcuts import redirect
from django.urls import reverse_lazy, reverse
from django.utils.translation import ugettext_lazy as _
from django.views.generic import UpdateView, ListView, CreateView, DetailView, DeleteView
from pragmatic.mixins import LoginPermissionRequiredMixin, DeleteObjectMixin
# from inventor.core.accounts.filters import UserFilter
from inventor.core.accounts.forms import UpdateProfileForm, UserForm
from inventor.core.accounts.models import User


class UserDetailView(LoginPermissionRequiredMixin, DetailView):
    model = User
    permission_required = 'accounts.view_user'


class UpdateProfileView(LoginRequiredMixin, UpdateView):
    model = User
    form_class = UpdateProfileForm
    template_name = 'accounts/user_profile_form.html'

    def get_object(self, queryset=None):
        return self.request.user

    def form_valid(self, form):
        messages.success(self.request, _('Profile successfully saved'))
        return super(UpdateProfileView, self).form_valid(form)

    def get_success_url(self):
        return self.request.user.get_absolute_url()


class UserUpdateView(LoginPermissionRequiredMixin, UpdateView):
    model = User
    form_class = UserForm
    permission_required = 'accounts.change_user'

    def form_valid(self, form):
        messages.success(self.request, _('User successfully saved'))
        return super(UserUpdateView, self).form_valid(form)

    def get_success_url(self):
        return self.object.get_absolute_url()


# class UserListView(LoginPermissionRequiredMixin, ListView):
#     model = User
#     paginate_by = 10
#     permission_required = 'accounts.list_user'
#
#     def get_queryset(self):
#         return self.filter.qs.select_related('team').prefetch_related('groups')
#
#     def dispatch(self, request, *args, **kwargs):
#         self.filter = UserFilter(request.GET, queryset=self.get_whole_queryset())
#         return super(UserListView, self).dispatch(request, *args, **kwargs)
#
#     def get_whole_queryset(self):
#         return User.objects.all()
#
#     def get_context_data(self, **kwargs):
#         context_data = super(UserListView, self).get_context_data(**kwargs)
#         context_data['filter'] = self.filter
#         return context_data


class UserCreateView(LoginPermissionRequiredMixin, CreateView):
    model = User
    template_name = 'accounts/user_create.html'
    form_class = UserForm
    permission_required = 'accounts.add_user'

    def form_valid(self, form):
        user = form.save()

        # generate random user password
        password = User.objects.make_random_password()
        user.set_password(password)

        # send password to user
        user.email_user(_('Welcome to TatryTravel.sk'), _('Your password is {}').format(password), fail_silently=False)

        messages.success(self.request, _('User successfully created'))
        return super(UserCreateView, self).form_valid(form)


class UserDeleteView(LoginPermissionRequiredMixin, DeleteObjectMixin, DeleteView):
    model = User
    success_url = reverse_lazy('accounts:user_list')
    template_name = 'confirm_delete.html'
    permission_required = 'accounts.delete_user'

    def dispatch(self, request, *args, **kwargs):
        if self.get_object() == self.request.user:
            messages.error(request, _("You can't delete your own account"))
            return redirect(reverse('accounts:user_list'))
        return super(UserDeleteView, self).dispatch(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        messages.success(request, _('User successfully deleted'))
        return super(UserDeleteView, self).delete(request, *args, **kwargs)



from django.db import models
from django.utils.translation import ugettext_lazy as _


class BookingMixin(models.Model):
    PERIODS = [  # TODO: deprecated? should be replaced by listing price?
        ('HOUR', _('hour')),
        ('DAY', _('day')),
        ('NIGHT', _('night')),
    ]
    booking_enabled = models.BooleanField(_('bookings enabled'), default=True)
    booking_url = models.URLField(_('booking URL'), max_length=300,
                                  blank=True)
    booking_period = models.CharField(_('booking period'), choices=PERIODS, max_length=5, blank=True)
    booking_min_period = models.SmallIntegerField(verbose_name=_('min period'),
                                                   blank=True, null=True, default=None)
    booking_max_period = models.SmallIntegerField(verbose_name=_('max period'),
                                                   blank=True, null=True, default=None)
    booking_min_persons = models.SmallIntegerField(verbose_name=_('min persons'),
                                                    blank=True, null=True, default=None)
    # booking_max_persons = models.SmallIntegerField(verbose_name=_('max persons'),
    #                                                 blank=True, null=True, default=None)
    capacity = models.SmallIntegerField(verbose_name=_('capacity'),
        blank=True, null=True, default=None)

    class Meta:
        abstract = True



from django.db.models import F
from django.views.generic import ListView, DetailView
from inventor.core.listings.filters import ListingFilter
from inventor.core.listings.models.general import Listing


class ListingListView(ListView):
    model = Listing
    filter_class = ListingFilter
    paginate_by = 12

    def dispatch(self, request, *args, **kwargs):
        self.filter = self.filter_class(data=request.GET, queryset=self.get_whole_queryset(), listing_type=None if self.model == Listing else self.model)
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        queryset = self.filter.qs.only('id', 'slug', 'title', 'location_id', 'location__title', 'image', 'price', 'price_unit', 'price_starts_at', 'promoted').annotate(location_title=F('location__title'))
        return queryset
        # return self.sort_queryset(queryset)

    def get_whole_queryset(self):
        return super().get_queryset().select_subclasses().order_by('-promoted', 'modified')

    def get_context_data(self, **kwargs):
        context_data = super().get_context_data(**kwargs)
        context_data.update({
            'title': self.model._meta.verbose_name_plural,
            'filter': self.filter
        })
        return context_data


class ListingDetailView(DetailView):
    model = Listing
    template_name = 'listings/listing_detail.html'

    # def get_queryset(self):
    #     return super().get_queryset().select_subclasses()

    def get_queryset(self):
        return super().get_queryset().only('id')

    def get_object(self, queryset=None):
        return super().get_object(queryset).get_real_instance()



from django.contrib.contenttypes.models import ContentType
from mptt.managers import TreeManager


class CategoryManager(TreeManager):
    def of_listing_type(self, listing_type):
        return self.filter(listing_type=ContentType.objects.get_for_model(listing_type))



import os
import shutil
from os.path import normpath

from django.core.management import call_command
from django.core.management.base import BaseCommand

from tenant.settings.common import project_root


class Command(BaseCommand):
    help = 'Resets listings migrations'

    def handle(self, *args, **options):
        call_command('migrate')
        self.reset_app_migrations(['lexicons', 'listings', 'bookings'])
        call_command('load_fixtures')

    def reset_app_migrations(self, apps):
        for app in apps:
            joined = project_root('source/inventor/core/{}/migrations/'.format(app))
            initial_migration_path = normpath(joined)

            if os.path.exists(initial_migration_path):
                print('Resetting migrations for app {}'.format(app))
                call_command('migrate', app, 'zero', '--fake')

        for app in apps:
            print('Deleting migrations for app {}'.format(app))

            joined = project_root('source/inventor/core/{}/migrations/'.format(app))
            initial_migration_path = normpath(joined)

            if os.path.exists(initial_migration_path):
                print('Deleting {}'.format(initial_migration_path))
                # os.remove(initial_migration_path)
                shutil.rmtree(initial_migration_path, ignore_errors=False, onerror=None)
            else:
                print('Migration {} does NOT exist!'.format(initial_migration_path))

        for app in apps:
            print('Setting migrations for app {}'.format(app))

            call_command('makemigrations', app)
            call_command('migrate', app, '--fake')



from django.contrib.contenttypes.models import ContentType
from django.core.management import call_command
from django.core.management.base import BaseCommand

from inventor.core.listings.models.general import Location, Category


class Command(BaseCommand):
    help = 'Loads sample data from fixtures'

    def handle(self, *args, **options):
        print('Loading fixtures...')
        call_command('loaddata',
                     'categories.json',  # TODO: update with values from deprecated accommodation and property types
                     'features.json',
                     'locations.json',
                     'accommodation_amenities.json',
                     #'accommodation_types.json',
                     #'property_types.json',
                     'listings.json'
                     )
        print('Fixtures loaded.')
        print('Setting lorem descriptions.')

        lexicons = ContentType.objects.filter(app_label__in=['lexicons', 'listings'])

        for lexicon in lexicons:
            model = lexicon.model_class()

            if hasattr(model, 'description'):
                model.objects.update(description={
                    'lexicons': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed nec enim condimentum, ullamcorper nulla consequat, ullamcorper enim. Nulla hendrerit ex a lacus finibus pulvinar. Sed sit amet urna tortor. Praesent efficitur mauris faucibus tempus hendrerit. Duis semper ipsum semper nibh ultricies, in posuere ex condimentum.',
                    'listings': "Aliquam malesuada luctus mauris, non ultrices purus consectetur vel. Mauris nulla sem, semper ut dui et, pulvinar elementum nisi. Sed sit amet sagittis ante. Duis porta velit eu congue gravida. Phasellus vehicula tortor arcu, quis fermentum purus placerat quis. In hac habitasse platea dictumst. Sed nisl orci, mattis et semper vel, facilisis et mi. Nulla faucibus libero vel velit aliquam finibus. Sed sollicitudin fringilla velit, a commodo turpis commodo eu.\n\nEtiam vulputate placerat magna. Nunc non sodales diam, malesuada auctor nisl. Ut lorem quam, auctor quis auctor nec, condimentum in purus. Donec aliquet tristique imperdiet. Duis rhoncus interdum lectus eget iaculis. Morbi elementum sapien libero, sed lacinia diam gravida in. Suspendisse in nibh magna. Aenean ex ex, viverra et neque ac, volutpat pharetra purus. Sed vel neque turpis. Phasellus eu hendrerit arcu, et rutrum magna. Nunc gravida, justo sed mollis feugiat, elit purus rutrum metus, at sagittis purus magna non purus. In hac habitasse platea dictumst."
                }.get(lexicon.app_label))

        print('Rebuilding MPTT trees...')
        Category.objects.rebuild()
        Location.objects.rebuild()
        print('MPTT trees rebuilded.')



from django.conf import settings
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import PermissionsMixin
from django.core.mail import send_mail
from django.db import models
from django.urls import reverse
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _
# from whistle.mixins import UserNotificationsMixin
from inventor.core.accounts.managers import UserManager, UserQuerySet
from inventor.core.accounts.utils import avatar_path_handler, avatar_thumbnail_exists, create_avatar_thumbnail


class User(AbstractBaseUser, PermissionsMixin):  # UserNotificationsMixin
    MAX_USERNAME_LENGTH = 30
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    first_name = models.CharField(_('first name'), max_length=30, blank=True, db_index=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True, db_index=True)
    email = models.EmailField(_('email'), unique=True)
    phone = models.CharField(_('phone'), max_length=30, blank=True)
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    date_joined = models.DateTimeField(_('date joined'), default=now)
    last_seen = models.DateTimeField(_('last seen'),
        blank=True, null=True, default=None)
    avatar = models.ImageField(
        verbose_name=_('avatar'),
        help_text=_('photo, image or icon'),
        max_length=1024,
        upload_to=avatar_path_handler,
        blank=True,
    )
    objects = UserManager()
    qs = UserQuerySet.as_manager()

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        ordering = ('first_name', 'last_name', 'email')
        default_permissions = settings.DEFAULT_PERMISSIONS

    def __str__(self):
        return self.get_full_name() or self.email

    def get_absolute_url(self):
        return reverse('accounts:user_detail', kwargs={'pk': self.pk})

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        """
        Returns the short name for the user.
        """
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def get_avatar_url(self, size):
        if self.avatar:
            if not avatar_thumbnail_exists(self, size):
                create_avatar_thumbnail(self, size)

            return self.avatar.storage.url(self.avatar_name(size))
        return settings.AVATAR_DEFAULT_URL

    def avatar_name(self, size):
        def find_extension(format):
            format = format.lower()

            if format == 'jpeg':
                format = 'jpg'

            return format

        ext = find_extension(settings.AVATAR_THUMB_FORMAT)

        return avatar_path_handler(
            instance=self,
            size=size,
            ext=ext
        )



from django.conf.urls import url
from django.utils.translation import pgettext_lazy
from inventor.core.accounts.views import UserDetailView, UpdateProfileView, UserUpdateView, \
    UserCreateView, UserDeleteView  # UserListView

app_name = 'accounts'

urlpatterns = [
    url(pgettext_lazy("url", r'^users/(?P<pk>[-\d]+)/$'), UserDetailView.as_view(), name='user_detail'),
    url(pgettext_lazy("url", r'^users/(?P<pk>[-\d]+)/update/$'), UserUpdateView.as_view(), name='user_update'),
    url(pgettext_lazy("url", r'^users/(?P<pk>[-\d]+)/delete/$'), UserDeleteView.as_view(), name='user_delete'),
    url(pgettext_lazy("url", r'^users/(?P<pk>[-\d]+)/changelog/$'), UserDetailView.as_view(template_name_suffix='_changelog'), name='user_changelog'),
    url(pgettext_lazy("url", r'^users/update/profile/$'), UpdateProfileView.as_view(), name='user_update_profile'),
    url(pgettext_lazy("url", r'^users/create/$'), UserCreateView.as_view(), name='user_create'),
    # url(pgettext_lazy("url", r'^users/$'), UserListView.as_view(), name='user_list'),
]



from PIL import Image
from django.core.files.base import ContentFile, File
import hashlib
import os
import six
from django.conf import settings
from django.core.cache import cache
from django.utils.text import slugify
from django.utils.encoding import force_bytes


def avatar_path_handler(instance=None, filename=None, size=None, ext=None):
    tmppath = ['avatars']
    #tmppath.append(force_text(instance.pk))

    if not filename:
        # Filename already stored in database
        filename = instance.avatar.name

    if size:
        tmppath.extend(['resized', str(size)])

    tmppath.append(os.path.basename(filename))
    return os.path.join(*tmppath)


cached_funcs = set()


def get_cache_key(user, size, prefix):
    """
    Returns a cache key consistent of a username and image size.
    """
    key = six.u('%s_%s_%s') % (prefix, user.pk, size)
    return six.u('%s_%s') % (slugify(key)[:100], hashlib.md5(force_bytes(key)).hexdigest())


def cache_result(default_size=settings.AVATAR_DEFAULT_SIZE):
    """
    Decorator to cache the result of functions that take a ``user`` and a
    ``size`` value.
    """
    if not settings.AVATAR_CACHE_ENABLED:
        def decorator(func):
            return func
        return decorator

    def decorator(func):
        def cached_func(user, size=None):
            prefix = func.__name__
            cached_funcs.add(prefix)
            key = get_cache_key(user, size or default_size, prefix=prefix)
            result = cache.get(key)
            if result is None:
                result = func(user, size or default_size)
                cache.set(key, result, settings.AVATAR_CACHE_TIMEOUT)
            return result
        return cached_func
    return decorator


def invalidate_cache(user, size=None):
    """
    Function to be called when saving or changing an user's avatars.
    """
    sizes = set(settings.AVATAR_AUTO_GENERATE_SIZES)
    if size is not None:
        sizes.add(size)
    for prefix in cached_funcs:
        for size in sizes:
            cache.delete(get_cache_key(user, size, prefix))


def avatar_thumbnail_exists(user, size):
    return user.avatar.storage.exists(user.avatar_name(size))


def create_avatar_thumbnail(user, size, quality=None):
    # invalidate the cache of the thumbnail with the given size first
    invalidate_cache(user, size)
    try:
        orig = user.avatar.storage.open(user.avatar.name, 'rb')
        image = Image.open(orig)
        quality = quality or settings.AVATAR_THUMB_QUALITY
        w, h = image.size
        if w != size or h != size:
            if w > h:
                diff = int((w - h) / 2)
                image = image.crop((diff, 0, w - diff, h))
            else:
                diff = int((h - w) / 2)
                image = image.crop((0, diff, w, h - diff))
            if image.mode not in ("RGB", "RGBA"):
                image = image.convert("RGB")
            image = image.resize((size, size), settings.AVATAR_RESIZE_METHOD)
            thumb = six.BytesIO()
            image.save(thumb, settings.AVATAR_THUMB_FORMAT, quality=quality)
            thumb_file = ContentFile(thumb.getvalue())
        else:
            thumb_file = File(orig)
        thumb = user.avatar.storage.save(user.avatar_name(size), thumb_file)
    except IOError:
        return  # What should we do here?  Render a "sorry, didn't work" img?



#!/usr/bin/env python
from setuptools import setup, find_packages


setup(
    name='django-inventor',
    version='0.2.0',
    description='Directory listing Django app',
    long_description=open('README.md').read(),
    author='Pragmatic Mates',
    author_email='info@pragmaticmates.com',
    maintainer='Pragmatic Mates',
    maintainer_email='info@pragmaticmates.com',
    url='https://github.com/PragmaticMates/django-inventor',
    packages=find_packages(),
    include_package_data=True,
    install_requires=('django-filter', 'django', 'python-pragmatic', 'django-pragmatic'),
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Development Status :: 3 - Alpha'
    ],
    license='GNU General Public License (GPL)',
    keywords="django directory listing catalog map",
)



from django.contrib.contenttypes.models import ContentType
from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _
from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel

from inventor.core.listings.mixins import SlugMixin
from .querysets import CategoryManager


class RegularLexicon(SlugMixin, models.Model):
    title = models.CharField(_('title'), max_length=100, unique=True)
    slug = models.SlugField(unique=True, max_length=SlugMixin.MAX_SLUG_LENGTH, default='')
    description = models.TextField(_('description'), blank=True)

    class Meta:
        abstract = True
        ordering = ('title',)

    def __str__(self):
        return self.title


class Category(SlugMixin, MPTTModel):
    title = models.CharField(_('title'), max_length=100)
    slug = models.SlugField(unique=True, max_length=SlugMixin.MAX_SLUG_LENGTH, default='')
    listing_type = models.ForeignKey(
        ContentType, verbose_name=_('listing type'), on_delete=models.SET_NULL,
        blank=True, null=True, default=None)
    parent = TreeForeignKey('self', on_delete=models.CASCADE, null=True, blank=True, related_name='children')
    objects = CategoryManager()

    class MPTTMeta:
        order_insertion_by = ['title']

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')
        ordering = ('title',)
        unique_together = (('title', 'listing_type'),)

    def __str__(self):
        return self.title


class Feature(SlugMixin, models.Model):
    title = models.CharField(_('title'), max_length=100, unique=True)
    slug = models.SlugField(unique=True, max_length=SlugMixin.MAX_SLUG_LENGTH, default='')
    # listing_type = models.ForeignKey(
    #     ContentType, verbose_name=_('listing type'), on_delete=models.PROTECT,
    #     blank=True, null=True, default=None)  # or many to many?

    class Meta:
        verbose_name = _('feature')
        verbose_name_plural = _('features')
        ordering = ('title',)

    def __str__(self):
        return self.title


class Location(SlugMixin, MPTTModel):
    title = models.CharField(_('title'), max_length=100, unique=True)
    slug = models.SlugField(unique=True, max_length=SlugMixin.MAX_SLUG_LENGTH, default='')
    description = models.TextField(_('description'), blank=True)
    parent = TreeForeignKey('self', on_delete=models.CASCADE, null=True, blank=True, related_name='children')

    class MPTTMeta:
        order_insertion_by = ['title']

    class Meta:
        verbose_name = _('location')
        verbose_name_plural = _('locations')
        ordering = ('title',)

    def __str__(self):
        return self.title


# Accommodation

class AccommodationAmenity(RegularLexicon):
    class Meta:
        verbose_name = _('accommodation amenity')
        verbose_name_plural = _('accommodation amenities')
        ordering = ('title',)


# TODO: Deprecated: replaced by categories

# class AccommodationType(RegularLexicon):
#     class Meta:
#         verbose_name = _('accommodation type')
#         verbose_name_plural = _('accommodation types')
#         ordering = ('title',)


# Property

# TODO: Deprecated: replaced by categories
# class PropertyType(RegularLexicon):
#     class Meta:
#         verbose_name = _('property type')
#         verbose_name_plural = _('property types')
#         ordering = ('title',)


from .signals import *  # TODO: move to AppConfig



from django.conf import settings
from django.contrib.gis.db import models
from django.core.validators import MinValueValidator, EMPTY_VALUES
from django.db import transaction
from django.utils.translation import ugettext_lazy as _

from inventor import settings as inventor_settings
from inventor.core.listings.managers import ListingQuerySet
from inventor.core.listings.models.general import Listing
from inventor.utils import generate_hash


class Booking(models.Model):
    HASH_KEY_LENGTH = 40
    STATUS_AWAITING_APPROVAL = 'AWAITING_APPROVAL'
    STATUS_PENDING_PAYMENT = 'PENDING_PAYMENT'
    STATUS_APPROVED = 'APPROVED'
    STATUS_CANCELLED = 'CANCELLED'
    STATUS_NO_SHOW = 'NO_SHOW'
    STATUS_COMPLETED = 'COMPLETED'
    STATUSES = [
        (STATUS_AWAITING_APPROVAL, _('Awaiting approval')),
        (STATUS_PENDING_PAYMENT, _('Pending payment')),
        (STATUS_APPROVED, _('Approved')),
        (STATUS_CANCELLED, _('Cancelled')),
        (STATUS_NO_SHOW, _('No-show')),
        (STATUS_COMPLETED, _('Completed')),
    ]
    status = models.CharField(verbose_name=_('status'), choices=STATUSES, max_length=17, default=STATUS_AWAITING_APPROVAL)
    traveler = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, verbose_name=_('traveler'),
                                 blank=True, null=True, default=None)
    listing = models.ForeignKey(Listing, on_delete=models.CASCADE, verbose_name=_('listing'))
    hash_key = models.CharField(verbose_name=_('hash key'), max_length=HASH_KEY_LENGTH, db_index=True, unique=True)
    book_from = models.DateTimeField(_('book from'), blank=True, null=True, default=None)
    book_to = models.DateTimeField(_('book to'), blank=True, null=True, default=None)
    number_of_persons = models.PositiveSmallIntegerField(_('number of persons'), blank=True, null=True, default=None)
    price = models.DecimalField(_('price'), help_text=inventor_settings.CURRENCY, max_digits=10, decimal_places=2, validators=[MinValueValidator(0)],
                                blank=True, null=True, default=None)
    message = models.TextField(_('message'), blank=True)

    # contact information
    person = models.CharField(_('person'), max_length=100, blank=True)
    phone = models.CharField(_('phone'), max_length=40, blank=True)
    email = models.EmailField(_('email'), blank=True)

    created = models.DateTimeField(_('created'), auto_now_add=True)
    modified = models.DateTimeField(_('modified'), auto_now=True)
    objects = ListingQuerySet.as_manager()

    class Meta:
        verbose_name = _('booking')
        verbose_name_plural = _('bookings')
        ordering = ('created',)

    def __str__(self):
        return f'#{self.id}'

    @staticmethod
    def generate_hash_key():
        while True:
            hash = generate_hash(length=Booking.HASH_KEY_LENGTH)

            if not Booking.objects.filter(hash_key=hash).exists():
                break

        return hash

    @transaction.atomic
    def save(self, *args, **kwargs):
        if self.hash_key in EMPTY_VALUES:
            self.hash_key = Booking.generate_hash_key()

        return super().save(*args, **kwargs)



from django.db.models import Q
from model_utils.managers import InheritanceQuerySet


class ListingQuerySet(InheritanceQuerySet):
    def without_points(self):
        return self.filter(point=None)

    def with_points(self):
        return self.exclude(point=None)

    def with_address(self):
        return self.exclude(Q(street='') | Q(city='') | Q(country=''))

    def published(self):
        return self.filter(published=True)

    def not_published(self):
        return self.filter(published=False)

    def promoted(self):
        return self.filter(promoted=True)

    def not_promoted(self):
        return self.filter(promoted=False)

    def by_keyword(self, keyword):
        return self.filter(
            Q(title__unaccent__icontains=keyword) |
            Q(description__unaccent__icontains=keyword)
        )



from django.conf import settings as django_settings


CURRENCY = getattr(django_settings, 'INVENTOR_CURRENCY', '€')
CURRENCY_AFTER_AMOUNT = getattr(django_settings, 'INVENTOR_CURRENCY_AFTER_AMOUNT', True)
PAGE_TITLE = getattr(django_settings, 'INVENTOR_PAGE_TITLE', 'Inventor')
PAGE_DESCRIPTION = getattr(django_settings, 'INVENTOR_PAGE_DESCRIPTION', 'Description')
PAGE_KEYWORDS = getattr(django_settings, 'INVENTOR_PAGE_KEYWORDS', 'Key words')
PAGE_APPLE_MOBILE_WEB_APP_TITLE = getattr(django_settings, 'INVENTOR_PAGE_APPLE_MOBILE_WEB_APP_TITLE', PAGE_TITLE)



from django.db.models.signals import post_save
from django.dispatch import receiver

from inventor.core.lexicons.models import Category


@receiver(post_save, sender=Category)
# @apm_custom_context('signals')
def update_children_listing_type(sender, instance, **kwargs):
    listing_type = instance.listing_type
    instance.get_descendants().update(listing_type=listing_type)
